@@ -21,8 +21,6 @@
 #endif
 
 #include <glib/gstdio.h>
-#include <X11/Xlib.h>
-#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <xfconf/xfconf.h>
 #include <libxfce4util/libxfce4util.h>
@@ -41,20 +39,24 @@
 
 
 /*
- * Private functions
+ * Plugin actions
  */
 
-static gboolean
-clipboard_manager_ownership_exists (void)
+static void
+plugin_action_set_text (GSimpleAction *action,
+                        GVariant *value,
+                        gpointer data)
 {
-  Display *display;
-  Atom atom;
-
-  display = gdk_x11_get_default_xdisplay ();
-  atom = XInternAtom (display, "CLIPBOARD_MANAGER", FALSE);
-  return XGetSelectionOwner (display, atom);
+  gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), g_variant_get_string (value, NULL), -1);
 }
 
+static const GActionEntry plugin_actions[] =
+{
+  { "set-text", plugin_action_set_text, "s", NULL, NULL },
+};
+
+
+
 /*
  * Plugin functions
  */
@@ -98,14 +100,11 @@ plugin_register (void)
   plugin = g_slice_new0 (MyPlugin);
   plugin->app = app;
   g_signal_connect_swapped (plugin->app, "activate", G_CALLBACK (plugin_popup_menu), plugin);
+  g_action_map_add_action_entries (G_ACTION_MAP (app), plugin_actions, G_N_ELEMENTS (plugin_actions), plugin);
   plugin->channel = xfconf_channel_new_with_property_base ("xfce4-panel", "/plugins/clipman");
 
   /* Daemon */
-  if (!clipboard_manager_ownership_exists ())
-    {
-      plugin->daemon = gsd_clipboard_manager_new ();
-      gsd_clipboard_manager_start (plugin->daemon, NULL);
-    }
+  plugin->daemon = xcp_clipboard_manager_get ();
 
   /* ClipmanActions */
   plugin->actions = clipman_actions_get ();
@@ -295,11 +294,7 @@ plugin_save (MyPlugin *plugin)
 void
 plugin_free (MyPlugin *plugin)
 {
-  if (plugin->daemon != NULL)
-    {
-      gsd_clipboard_manager_stop (plugin->daemon);
-      g_object_unref (plugin->daemon);
-    }
+  g_object_unref (plugin->daemon);
   gtk_widget_destroy (plugin->menu);
   g_object_unref (plugin->channel);
   g_object_unref (plugin->actions);
@@ -309,7 +304,8 @@ plugin_free (MyPlugin *plugin)
 #ifdef PANEL_PLUGIN
   gtk_widget_destroy (plugin->button);
 #elif defined (STATUS_ICON)
-  gtk_widget_destroy (plugin->popup_menu);
+  if (plugin->popup_menu != NULL)
+    gtk_widget_destroy (plugin->popup_menu);
 #endif
 
   g_object_unref (plugin->app);
@@ -343,7 +339,7 @@ plugin_about (MyPlugin *plugin)
                          "version", PACKAGE_VERSION,
                          "copyright", "Copyright © 2003-2023 The Xfce development team",
                          "license", license,
-                         "website", "https://docs.xfce.org/panel-plugins/xfce4-clipman-plugin",
+                         "website", PACKAGE_URL,
                          "website-label", "docs.xfce.org",
                          "authors", authors,
                          "documenters", documenters,
@@ -369,37 +365,56 @@ plugin_configure (MyPlugin *plugin)
   }
 }
 
-static void
-plugin_popup_menu_real (GtkClipboard *clipboard,
-                        const gchar  *text,
-                        gpointer      data)
+void
+plugin_popup_menu (MyPlugin *plugin)
 {
-  MyPlugin *plugin = data;
+  GdkEvent *event = gtk_get_current_event ();
+  gboolean popup_command = (event == NULL);
 
-  /* store primary selection for later use */
+  if (popup_command)
+    {
+      GdkSeat *seat = gdk_display_get_default_seat (gdk_display_get_default ());
+      event = gdk_event_new (GDK_BUTTON_PRESS);
+      event->button.window = g_object_ref (gdk_get_default_root_window ());
+      gdk_event_set_device (event, gdk_seat_get_pointer (seat));
+    }
+
+  /* store clipboard contents for later use when the menu is shown: we can't do this
+   * at that time as it iterates the main loop and it causes critical warnings */
+  g_object_set_data_full (G_OBJECT (plugin->menu), "selection-clipboard",
+                          gtk_clipboard_wait_for_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD)), g_free);
   g_object_set_data_full (G_OBJECT (plugin->menu), "selection-primary",
-                          g_strdup (text), g_free);
+                          gtk_clipboard_wait_for_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY)), g_free);
 
   if (xfconf_channel_get_bool (plugin->channel, "/tweaks/popup-at-pointer", FALSE))
     {
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
-                      NULL, NULL,
-                      0, gtk_get_current_event_time ());
-G_GNUC_END_IGNORE_DEPRECATIONS
+#ifdef PANEL_PLUGIN
+      if (!popup_command)
+        {
+          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->button), TRUE);
+          xfce_panel_plugin_register_menu (plugin->panel_plugin, GTK_MENU (plugin->menu));
+        }
+#endif
+      gtk_menu_popup_at_pointer (GTK_MENU (plugin->menu), event);
     }
   else
     {
 #ifdef PANEL_PLUGIN
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->button), TRUE);
+#if LIBXFCE4PANEL_CHECK_VERSION (4, 17, 2)
+      xfce_panel_plugin_popup_menu (plugin->panel_plugin, GTK_MENU (plugin->menu), plugin->button, event);
+#else
+      xfce_panel_plugin_register_menu (plugin->panel_plugin, GTK_MENU (plugin->menu));
       gtk_menu_set_screen (GTK_MENU (plugin->menu), gtk_widget_get_screen (plugin->button));
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
                       plugin->menu_position_func, plugin,
                       0, gtk_get_current_event_time ());
 G_GNUC_END_IGNORE_DEPRECATIONS
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->button), TRUE);
-      xfce_panel_plugin_register_menu (plugin->panel_plugin, GTK_MENU (plugin->menu));
+#endif
+
 #elif defined (STATUS_ICON)
+
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       gtk_menu_set_screen (GTK_MENU (plugin->menu),
                            gtk_status_icon_get_screen (plugin->status_icon));
@@ -408,30 +423,7 @@ G_GNUC_BEGIN_IGNORE_DEPRECATIONS
                       0, gtk_get_current_event_time ());
 G_GNUC_END_IGNORE_DEPRECATIONS
 #endif
-  }
-}
+    }
 
-static void
-plugin_popup_menu_primary (GtkClipboard *clipboard,
-                           const gchar  *text,
-                           gpointer      data)
-{
-  MyPlugin *plugin = data;
-
-  /* store clipboard selection for later use */
-  g_object_set_data_full (G_OBJECT (plugin->menu), "selection-clipboard",
-                          g_strdup (text), g_free);
-
-  /* request primary selection */
-  gtk_clipboard_request_text (gtk_clipboard_get (GDK_SELECTION_PRIMARY),
-                              plugin_popup_menu_real, plugin);
-}
-
-void
-plugin_popup_menu (MyPlugin *plugin)
-{
-  /* first, request text of each clipboard asynchronously, to not dispatch
-   * current event associated to menu popup */
-  gtk_clipboard_request_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD),
-                              plugin_popup_menu_primary, plugin);
+  gdk_event_free (event);
 }
