@@ -23,15 +23,12 @@
 #endif
 
 #include <gtk/gtk.h>
-#include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
-#include <X11/extensions/XTest.h>
 
 #include <libxfce4ui/libxfce4ui.h>
 #include <libxfce4util/libxfce4util.h>
-#include <x11-clipboard-manager/daemon.h>
 
-#include <menu.h>
+#include "common.h"
 #include <plugin.h>
 #include <history.h>
 
@@ -63,6 +60,8 @@ clipman_history_row_activated (GtkTreeView       *treeview,
   GtkTreeSelection *selection;
   GtkTreeModel *model;
   GtkTreeIter iter;
+  GDBusProxy *proxy;
+  GError *error = NULL;
   gboolean ret;
   gchar *text;
 
@@ -77,11 +76,45 @@ clipman_history_row_activated (GtkTreeView       *treeview,
   gtk_tree_model_get (model, &iter,
                       COLUMN_TEXT, &text,
                       -1);
-  g_object_set_data_full (G_OBJECT (plugin->app), "text", text, g_free);
+
+  /* set clipboard text from the daemon instance */
+  proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
+                                         G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES
+                                         | G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
+                                         NULL, "org.xfce.clipman", "/org/xfce/clipman",
+                                         "org.gtk.Actions", NULL, &error);
+  if (proxy != NULL)
+    {
+      GVariantBuilder *builder1 = g_variant_builder_new (G_VARIANT_TYPE ("av"));
+      GVariantBuilder *builder2 = g_variant_builder_new (G_VARIANT_TYPE ("a{sv}"));
+      GVariant *value;
+
+      g_variant_builder_add (builder1, "v", g_variant_new_string (text));
+      value = g_dbus_proxy_call_sync (proxy, "org.gtk.Actions.Activate",
+                                      g_variant_new ("(sava{sv})", "set-text", builder1, builder2),
+                                      G_DBUS_CALL_FLAGS_NONE, -1, NULL, &error);
+      g_variant_builder_unref (builder1);
+      g_variant_builder_unref (builder2);
+
+      if (value != NULL)
+        g_variant_unref (value);
+      else
+        {
+          g_warning ("Failed to activate set-text action on org.xfce.clipman: %s", error->message);
+          g_error_free (error);
+        }
+    }
+  else
+    {
+      g_warning ("Failed to get D-Bus proxy for org.xfce.clipman: %s", error->message);
+      g_error_free (error);
+    }
 
   window = gtk_widget_get_toplevel (GTK_WIDGET (treeview));
   if (GTK_IS_WINDOW (window))
     gtk_dialog_response (GTK_DIALOG (window), GTK_RESPONSE_CLOSE);
+
+  g_free (text);
 }
 
 static void
@@ -248,12 +281,6 @@ clipman_history_treeview_init (MyPlugin *plugin)
   g_signal_connect_swapped (G_OBJECT (entry), "changed", G_CALLBACK (clipman_history_treeview_filter_and_select), plugin);
   g_signal_connect (G_OBJECT (entry), "activate", G_CALLBACK (clipman_history_search_entry_activate), plugin);
 
-  if (internal_paste_on_activate != PASTE_INACTIVE)
-    {
-      g_signal_connect (G_OBJECT (entry), "key-press-event", G_CALLBACK (clipman_history_key_event), plugin);
-      g_signal_connect (G_OBJECT (entry), "key-release-event", G_CALLBACK (clipman_history_key_event), plugin);
-    }
-
   /* Create the treeview */
   plugin->treeview = treeview = gtk_tree_view_new_with_model (filter);
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
@@ -261,12 +288,6 @@ clipman_history_treeview_init (MyPlugin *plugin)
   g_signal_connect_swapped (G_OBJECT (treeview), "start-interactive-search", G_CALLBACK (gtk_widget_grab_focus), entry);
   g_signal_connect (G_OBJECT (treeview), "row-activated", G_CALLBACK (clipman_history_row_activated), plugin);
 
-  if (internal_paste_on_activate != PASTE_INACTIVE)
-    {
-      g_signal_connect (G_OBJECT (treeview), "key-press-event", G_CALLBACK (clipman_history_key_event), plugin);
-      g_signal_connect (G_OBJECT (treeview), "key-release-event", G_CALLBACK (clipman_history_key_event), plugin);
-    }
-
   gtk_container_add (GTK_CONTAINER (scroll), treeview);
   gtk_widget_show (treeview);
 
@@ -365,12 +386,12 @@ clipman_history_copy_or_paste_on_activate (MyPlugin *plugin,
   if (paste_on_activate != PASTE_INACTIVE)
     {
       icon = gtk_image_new_from_icon_name ("edit-paste-symbolic", GTK_ICON_SIZE_BUTTON);
-      button_text = g_strdup_printf (_("_Paste"));
+      button_text = _("_Paste");
     }
   else
     {
       icon = gtk_image_new_from_icon_name ("edit-copy-symbolic", GTK_ICON_SIZE_BUTTON);
-      button_text = g_strdup_printf (_("_Copy"));
+      button_text = _("_Copy");
     }
 
   gtk_button_set_image (GTK_BUTTON (plugin->submit_button), icon);
@@ -380,8 +401,19 @@ clipman_history_copy_or_paste_on_activate (MyPlugin *plugin,
 static void
 clipman_history_paste_on_activate_changed (MyPlugin *plugin)
 {
-  internal_paste_on_activate = xfconf_channel_get_uint (plugin->channel, "/tweaks/paste-on-activate", PASTE_INACTIVE);
+  internal_paste_on_activate = xfconf_channel_get_uint (plugin->channel, "/tweaks/paste-on-activate", PASTE_INACTIVE)
+                               && GDK_IS_X11_DISPLAY (gdk_display_get_default ());
   clipman_history_copy_or_paste_on_activate (plugin, internal_paste_on_activate);
+
+  g_signal_handlers_disconnect_by_func (plugin->entry, clipman_history_key_event, plugin);
+  g_signal_handlers_disconnect_by_func (plugin->treeview, clipman_history_key_event, plugin);
+  if (internal_paste_on_activate != PASTE_INACTIVE)
+    {
+      g_signal_connect (G_OBJECT (plugin->entry), "key-press-event", G_CALLBACK (clipman_history_key_event), plugin);
+      g_signal_connect (G_OBJECT (plugin->entry), "key-release-event", G_CALLBACK (clipman_history_key_event), plugin);
+      g_signal_connect (G_OBJECT (plugin->treeview), "key-press-event", G_CALLBACK (clipman_history_key_event), plugin);
+      g_signal_connect (G_OBJECT (plugin->treeview), "key-release-event", G_CALLBACK (clipman_history_key_event), plugin);
+    }
 }
 
 GtkWidget *
@@ -422,7 +454,6 @@ clipman_history_dialog_init (MyPlugin *plugin)
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_APPLY);
 #endif
   gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (plugin->submit_button)), "suggested-action");
-  clipman_history_paste_on_activate_changed (plugin);
 
   g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (clipman_history_dialog_response), plugin);
 
@@ -496,6 +527,9 @@ clipman_history_command_line (GApplication *app,
   gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (plugin->dialog))), box);
   gtk_widget_show_all (box);
 
+  /* setup paste-on-activate */
+  clipman_history_paste_on_activate_changed (plugin);
+
   gtk_widget_show_all (plugin->dialog);
 
   return EXIT_SUCCESS;
@@ -530,8 +564,6 @@ main (gint argc, gchar *argv[])
 {
   GtkApplication *app;
   int status;
-  GtkClipboard *clipboard;
-  const gchar *text;
 
   /* Setup translation domain */
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
@@ -548,27 +580,11 @@ main (gint argc, gchar *argv[])
   app = gtk_application_new ("org.xfce.clipman.history", G_APPLICATION_HANDLES_COMMAND_LINE);
   g_signal_connect (app, "command-line", G_CALLBACK (clipman_history_command_line), NULL);
   status = g_application_run (G_APPLICATION (app), argc, argv);
-  text = g_object_get_data (G_OBJECT (app), "text");
-
-  /* doing copy and paste outside of g_main to assure window being already gone and
-   * as in some occasions the implicit gtk_clipboard_store() lags for 10 seconds
-   */
-  if (text != NULL)
-    {
-      clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
-      gtk_clipboard_set_text (clipboard, text, -1);
-
-      /* we are out of the g_main - so we have store() on our own */
-      gtk_clipboard_store (clipboard);
-
-      /* Should be after gtk_widget_destroy() to assure focus being in the (previous) "target" window */
-      if (internal_paste_on_activate != PASTE_INACTIVE)
-        {
-          cb_paste_on_activate (internal_paste_on_activate);
-        }
-    }
-
   g_object_unref (app);
 
+  /* Should be after gtk_widget_destroy() to assure focus being in the (previous) "target" window */
+  if (internal_paste_on_activate != PASTE_INACTIVE)
+    clipman_common_paste_on_activate (internal_paste_on_activate);
+
   return status;
 }
