@@ -91,6 +91,8 @@ prop_dialog_init (void)
 #else
   gtk_widget_hide(GTK_WIDGET (gtk_builder_get_object (builder, "show-qr-code")));
 #endif
+  if (!GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+    gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (builder, "persistent-selections")));
 
   /* paste-on-activate combobox */
   combobox = GTK_WIDGET (gtk_builder_get_object (builder, "combobox-paste-on-activate"));
@@ -100,6 +102,8 @@ prop_dialog_init (void)
   /* TRANSLATORS: Keyboard shortcut */
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combobox), _("Shift+Insert"));
   gtk_combo_box_set_active (GTK_COMBO_BOX (combobox), 0);
+  if (!GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+    gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (builder, "hbox1")));
 
 #ifdef HAVE_QRENCODE
   xfconf_g_property_bind (xfconf_channel, "/settings/show-qr-code", G_TYPE_BOOLEAN,
@@ -133,6 +137,8 @@ prop_dialog_init (void)
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combobox), _("shows actions"));
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combobox), _("hides actions"));
   gtk_combo_box_set_active (GTK_COMBO_BOX (combobox), 0);
+  if (!GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+    gtk_widget_hide (GTK_WIDGET (gtk_builder_get_object (builder, "hbox-skip-action")));
 
   xfconf_g_property_bind (xfconf_channel, "/settings/enable-actions", G_TYPE_BOOLEAN,
                           gtk_builder_get_object (builder, "box-actions"), "sensitive");
@@ -272,7 +278,7 @@ cb_show_help (GtkButton *button)
   else
     {
       g_free (docpath);
-      docpath = g_strdup ("http://docs.xfce.org/panel-plugins/clipman/start");
+      docpath = g_strdup (PACKAGE_URL);
     }
 
   /* Open documentation in webbrowser */
@@ -939,7 +945,7 @@ command_line (GApplication *app,
   actions = clipman_actions_get ();
   prop_dialog_init ();
   gtk_window_set_application (GTK_WINDOW (settings_dialog), GTK_APPLICATION (app));
-  gtk_widget_show_all (settings_dialog);
+  gtk_widget_show (settings_dialog);
 
   return EXIT_SUCCESS;
 }
